<?php
// +----------------------------------------------------------------------
// | 登录处理
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

namespace app\admin\controller\Login;

use app\common\controller\_Admin;
use app\admin\model\AuthAdmin as Admin;
use think\facade\Session;
use think\facade\Cookie;
use think\facade\Request;

class handle extends _Admin
{

    // 默认留空即可
    protected function initialize()
    {
        parent::initialize();
        // 启动模板布局
        $this->view->engine->layout(false);
    }

    // 统一返回方法
    // $param 必留，用于接收get、post的参数传递
    public function run($param=[], $_this) {
        $status = Request::instance()->post('remember');

        $vif = $this->_where($param);
        if ($vif !== true) return $vif;
        
        return self::_json('00', '登录成功~！');
    }

    // 条件过滤
    private function _where($param=[]) {
        if (empty($param['username']) || empty($param['password'])) return self::_json('01', '用户名或密码错误');

        $info = Admin::alias('A')
        ->join('auth_role B', 'A.role = B.id', 'LEFT')
        ->field('A.*, B.menu_list, B.status as role_status')
        ->where('A.name', $param['username'])
        ->whereOr('A.phone', $param['username'])
        ->find();

        if (!$info) return self::_json('01', '用户名或密码错误');
        if ($info['status'] == 0) return self::_json('01', '该账号已被禁用！');
        if (self::isset_empty($info, 'role_status') && $info['role_status'] == 0) return self::_json('01', '该账号已被禁用！');
        if (md5($param['password'].strtotime($info['create_time'])) != $info['pwd']) return self::_json('01', '用户名或密码错误');
        
        # 记住登录状态
        Session::set('admin', $info);

        // 七天登录
        $this->_auto($param, $info['id']);
        // 记录登录日志
        $this->_login_log($info['id']);
        return true;
    }

    // 7天自动登录
    private function _auto($param=[], $id) {
        # 七天自动登录
        if (self::isset_empty($param, 'remember') && $param['remember'] == 1) {
            $ip   = Request::instance()->ip();  // IP设备绑定
            $time = time() + (86400 * 7);       // 过期时间
            $rand = mt_rand(100000, 999999); // 随机安全码
            # 使用密码本加密用户名，使用SHA1加密签名串
            $sha1 = \org\Encryption::run($param['username']).'|'.sha1($rand.$time.$ip);
            # 七天登录
            Cookie::set('login_auto', $sha1, (86400 * 7));
            # 把三要素写入管理员表
            Admin::where('id', $id)->update([
                'auto_time' => $time,
                'auto_rand' => $rand,
                'auto_ip'   => $ip,
            ]);
        }
    }

    // 登录日志
    private function _login_log($id) {
        $request = Request::instance();
        $ip      = $request->ip();
        $data    = \org\Ipcity::run($ip);
		$data    = [
            'ip'   => $ip,
            'admin' => $id,
            'province' => $data['province'] ?: '无',
            'city' => $data['city'] ?: '无',
            'area' => $data['area'] ?: '无',
			'create_time' => time(),
		];
		\think\Db::name('auth_admin_login')->insert($data);
    }
    
}
