<?php
// +----------------------------------------------------------------------
// | WebSocket启动入口
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

namespace app\socket\controller;

use think\Controller;
use think\facade\Config;

class Go extends Controller{
    /**
	 * WS的启动实例
	*/
	public $_ws;
	/**
	 * 配置项
	*/
	public $_config;
    
    /**
     * 启动入口
     * @todo 无
     * @author 小黄牛
     * @version v1.0.9 + 2020.04.16
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    public function start() {
        # 读取配置
        $this->_config = Config::pull('laf');

        # WSS配置
        $wss = SWOOLE_SOCK_TCP;
        if (!empty($this->_config['websocket_crt_path'])) {
            $wss = SWOOLE_SOCK_TCP | SWOOLE_SSL;
        }
        # 创建websocket服务器对象，监听0.0.0.0:9502端口
        $this->_ws = new \swoole_websocket_server($this->_config['websocket_host'], $this->_config['websocket_port'], SWOOLE_PROCESS, $wss);
        # 配置项
        $set = [];
        # 屏蔽后可根据CPU，自动识别进程数
        if (!empty($this->_config['websocket_max_process'])) {
            $set['worker_num'] = $this->_config['websocket_max_process'];
        }
        # WSS配置
        if (!empty($this->_config['websocket_crt_path'])) {
            $set['ssl_cert_file'] = $this->_config['websocket_crt_path'];
            $set['ssl_key_file'] = $this->_config['websocket_key_path'];
        }
        $this->_ws->set($set);
        # 内存表创建
        $this->ram_table();
        # 打开redis
        $this->create_redis();
        # 配置也放到实例里
        $this->_ws->config = $this->_config;
        # 载入事件流程
        $this->load_event();
    }
    
    /**
     * 创建内存表
     * @todo 无
     * @author 小黄牛
     * @version v1.1.1 + 2020.04.23
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    private function ram_table() {
        # 创建用户在线表状态
        $user = new \swoole_table($this->_config['websocket_max_online']);
        # 创建字段
        $user->column('fd', \swoole_table::TYPE_INT, 8); // FD
        $user->column('status', \swoole_table::TYPE_INT, 8); // 状态
        $user->column('heartbeat', \swoole_table::TYPE_INT, 8); // 心跳重连数
        $user->column('id', \swoole_table::TYPE_STRING, 32); // 会员ID
        $user->create();
        $this->_ws->user = $user;
    }

    /**
     * 创建Redis
     * @todo 无
     * @author 小黄牛
     * @version v1.1.1 + 2020.04.23
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    private function create_redis() {
        # 创建用户实时在线表
        $redis = new \Redis();
        $redis->connect($this->_config['websocket_redis_host'], $this->_config['websocket_redis_port']);
        if ($this->_config['websocket_redis_pwd']) {
            $redis->auth($this->_config['websocket_redis_pwd']);
        }
        $this->_ws->redis = $redis;
        if (!$redis->ping()) {
            die('redis链接失败：'.$redis->ping());
        }
    }

    /**
     * 载入服务事件
     * @todo 无
     * @author 小黄牛
     * @version v1.1.1 + 2020.04.23
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    private function load_event() {
        # 监听外部调用请求
        $this->_ws->on('request', [new \app\socket\controller\event\onRequest($this->_ws), 'run']);
        # 监听进程启动事件
        $this->_ws->on('Start', [new \app\socket\controller\event\onStart($this->_ws), 'run']);
        # 监听WebSocket握手申请
        $this->_ws->on('open', [new \app\socket\controller\event\onOpen($this->_ws), 'run']);
        # 监听客户端消息发送请求
        $this->_ws->on('message', [new \app\socket\controller\event\onMessage($this->_ws), 'run']);
        # 监听客户端退出事件
        $this->_ws->on('close', [new \app\socket\controller\event\onClose($this->_ws), 'run']);
        # 监听进程关闭事件，用来做关服业务
        $this->_ws->on('Shutdown', [new \app\socket\controller\event\onShutdown($this->_ws), 'run']);
        # 监听进程重启事件
        $this->_ws->on('WorkerStart', [new \app\socket\controller\event\onWorkerStart($this->_ws), 'run']);
        # 启动websocket
        $this->_ws->start();
    }
}
