<?php
// +----------------------------------------------------------------------
// | 微信用户相关
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

namespace wx\sdk;

use wx\Weixins;
use think\facade\Request;

class union extends Weixins{
    
    /**
     * 获取用户列表
     * @todo 无
     * @author 小黄牛
     * @version v1.0.8 + 2020.04.10
     * @deprecated 暂不弃用
     * @global 无
     * @param string $open_id 第一个拉取的OPENID，不填默认从头开始拉取
     * @return void
    */
	public static function get_user_list($open_id=null){
        $actoken = \wx\sdk\access_token::get();

        $url   = self::str_url(self::$config['user_list'], [$actoken, $open_id]);
        $json  = self::https_curl($url);
        $res   = json_decode($json, true);

        if (isset($res['errcode'])) {
            \log\Log::init()->insert('请求错误：'.$json, 'error');
            return false;
        } else {
            \log\Log::init()->insert('请求成功：'.$json);
        }

        return $res;
    }

    /**
     * 设置用户的备注名
     * @todo 无
     * @author 小黄牛
     * @version v1.0.8 + 2020.04.10
     * @deprecated 暂不弃用
     * @global 无
     * @param string $open_id 用户的OpenId
	 * @param string $remark 设置的备注名
     * @return bool
    */
	public static function remark($open_id, $remark) {
        $actoken = \wx\sdk\access_token::get();
        # 使用OpenID去获取用户信息
        $url   = self::str_url(self::$config['remarks'], [$actoken]);
        $json  = self::https_curl($url, json_encode([
            'openid' => $open_id,
            'remark' => $remark
        ]));
        $res   = json_decode($json, true);

        if ($res['errcode'] != 0) {
            \log\Log::init()->insert('请求错误：'.$json, 'error');
            return false;
        } else {
            \log\Log::init()->insert('请求成功：'.$json);
        }

        return $res;
    }
    
    /**
     * 生成网页登录授权链接
     * @todo 无
     * @author 小黄牛
     * @version v1.0.8 + 2020.04.10
     * @deprecated 暂不弃用
     * @global 无
     * @param string $url 授权后重定向的回调链接地址
     * @param string $type 授权方式 base | userinfo
	 * @param string $state 自己定义的参数，具体看微信开发手册
     * @return string
    */
	public static function login($url, $type='base', $state=1){
        self::get_config();
        $status = 'snsapi_'.$type;

        $array = [self::$config['appid'], urlencode($url), $status, $state];
        
        return self::str_url(self::$config['snsapi'], $array);
    }

    /**
     * 根据code去获取特殊access_token
     * @todo 无
     * @author 小黄牛
     * @version v1.0.8 + 2020.04.10
     * @deprecated 暂不弃用
     * @global 无
     * @param string $code 授权后返回的code参数
     * @return array|bool
    */
	public static function token($code=''){
        self::get_config();
        if (empty($code)) {
            $code = $_GET['code'];
        }
        $url = self::str_url(self::$config['snsapi_access_token'], [
            self::$config['appid'],
			self::$config['appsecret'],
			$code
        ]);
        $json  = self::https_curl($url);
        $res   = json_decode($json, true);

        if (isset($res['errcode'])) {
            \log\Log::init()->insert('请求错误：'.$json, 'error');
            return false;
        } else {
            \log\Log::init()->insert('微信浏览器登录，使用code获取access_token：'.$json);
        }
        return $res;
    }

    /**
     * 根据凭据，更新特殊access_token
     * @todo 注意，在get_code_token()中并没有返回refresh_token参数，所以需要自行修改代码获取
     * @author 小黄牛
     * @version v1.0.8 + 2020.04.10
     * @deprecated 暂不弃用
     * @global 无
     * @param string $refresh_token 在token()中的更新凭据
     * @return array|bool
    */
	public static function save_token($refresh_token){
        self::get_config();
        $url = self::str_url(self::$config['snsapi_update_access_token'], [
            self::$config['appid'],
			$refresh_token
        ]);
        $json  = self::https_curl($url);
        $res   = json_decode($json, true);

        if (isset($res['errcode'])) {
            \log\Log::init()->insert('请求错误：'.$json, 'error');
            return false;
        } else {
            \log\Log::init()->insert('微信浏览器登录，使用refresh_token更新后获取的access_token'.$json);
        }

		return $res;
    }

    /**
     * 确认授权：使用特殊access_token去获取用户信息
     * @todo 无
     * @author 小黄牛
     * @version v1.0.8 + 2020.04.10
     * @deprecated 暂不弃用
     * @global 无
     * @param string $token 特殊access_token
	 * @param string $open_id 用户的OpenId
     * @return array|bool 微信用户的基本信息
    */
	public static function userinfo($token, $open_id){
        self::get_config();

        # 确认授权的获取方式
        $url = self::str_url(self::$config['snsapi_user_info'], [
            $token,
			$open_id,
			self::$config['web_lang']
        ]);
        $json  = self::https_curl($url);
        $res   = json_decode($json, true);
        if (isset($res['errcode'])) {
            \log\Log::init()->insert('请求错误：'.$json, 'error');
            return false;
        } else {
            \log\Log::init()->insert('微信浏览器登录，点击授权：使用特殊actoken获取到的用户信息：'.$json);
        }

		# 返回点击授权的用户信息
		return $res;
    }

    /**
     * 静默授权：使用openid去获取用户信息
     * @todo 无
     * @author 小黄牛
     * @version v1.0.8 + 2020.04.10
     * @deprecated 暂不弃用
     * @global 无
     * @param string $open_id 用户的OpenId
     * @return array|bool 微信用户的基本信息
    */
    public static function basic($open_id) {
        # 发送请求
        $actoken = \wx\sdk\access_token::get();
        # 使用OpenID去获取用户信息
        $url   = self::str_url(self::$config['basic'], [$actoken, $open_id]);
        $json  = self::https_curl($url);
        $res   = json_decode($json, true);
        if (isset($res['errcode'])) {
            \log\Log::init()->insert('请求错误：'.$json, 'error');
            return false;
        } else {
            \log\Log::init()->insert('微信浏览器登录，静默授权：使用普通actoken获取到的用户信息：'.$json);
        }

		# 返回点击授权的用户信息
        return $res;
    }
}